@@ -3,34 +3,42 @@ import time
 import traceback
 
 from PyQt5 import QtGui
-from PyQt5.QtCore import (Qt, pyqtSignal)
+from PyQt5.QtCore import (Qt, pyqtSignal, QObject)
 from PyQt5.QtGui import QColor
 from PyQt5.QtWidgets import QMenu, QFileDialog, QTreeWidget, QTreeWidgetItem, \
-	QTreeWidgetItemIterator, QMessageBox
+	QTreeWidgetItemIterator, QMessageBox, QAction
 
+import pre_workbench.app
 from pre_workbench.configs import SettingsField
-from pre_workbench import configs
-from pre_workbench.genericwidgets import showSettingsDlg
-from pre_workbench.algo.rangelist import Range
+from pre_workbench import configs, guihelper
+from pre_workbench.controls.genericwidgets import showSettingsDlg
+from pre_workbench.algo.range import Range
 
-from pre_workbench.structinfo.format_info import FormatInfo, StructFI, VariantStructFI, SwitchFI, RepeatStructFI
+from pre_workbench.structinfo.format_info import FormatInfo, StructFI, VariantStructFI, SwitchFI, RepeatStructFI, \
+	UnionFI, BitStructFI
 from pre_workbench.structinfo.parsecontext import FormatInfoContainer
 from pre_workbench.structinfo.serialization import deserialize_fi
-from pre_workbench.textfile import showScintillaDialog
+from pre_workbench.controls.scintillaedit import showScintillaDialog
 from pre_workbench.typeeditor import showTypeEditorDlg, showTreeEditorDlg
 from pre_workbench.util import PerfTimer
 
 
-class InteractiveFormatInfoContainer(FormatInfoContainer):
-	def __init__(self, parent, **kw):
-		super().__init__(**kw)
-		self.parent = parent
+class InteractiveFormatInfoContainer(QObject, FormatInfoContainer):
+	updated = pyqtSignal()
+
+	def __init__(self, **kw):
+		QObject.__init__(self)
+		FormatInfoContainer.__init__(self, **kw)
+
+	def write_file(self, fileName):
+		super().write_file(fileName)
+		self.updated.emit()
 
 	def get_fi_by_def_name(self, def_name):
 		try:
 			return self.definitions[def_name]
 		except KeyError:
-			if QMessageBox.question(self.parent, "Format Info", "Reference to undefined formatinfo '"+def_name+"'. Create it now?") == QMessageBox.Yes:
+			if QMessageBox.question(pre_workbench.app.MainWindow, "Format Info", "Reference to undefined formatinfo '" + def_name + "'. Create it now?") == QMessageBox.Yes:
 				params = showTypeEditorDlg("format_info.tes", "AnyFI", title="Create formatinfo '"+def_name+"'")
 				if params is None: raise
 				self.definitions[def_name] = deserialize_fi(params)
@@ -43,8 +51,8 @@ class InteractiveFormatInfoContainer(FormatInfoContainer):
 class RangeTreeWidget(QTreeWidget):
 	def __init__(self, parent=None):
 		super().__init__(parent)
-		self.itemActivated.connect(self.fiTreeItemActivated)
-		self.setColumnCount(5)
+		self.itemActivated.connect(self._fiTreeItemActivated)
+		self.setColumnCount(6)
 		self.setColumnWidth(0, 400)
 		self.headerItem().setText(0, "Grammar Tree")
 		self.headerItem().setText(1, "Offset")
@@ -52,30 +60,31 @@ class RangeTreeWidget(QTreeWidget):
 		self.setColumnWidth(3, 200)
 		self.headerItem().setText(3, "Displayed Value")
 		self.headerItem().setText(4, "Raw Value")
+		self.headerItem().setText(5, "Print")
 		self.header().moveSection(3, 1)
 		self.setContextMenuPolicy(Qt.CustomContextMenu)
-		self.customContextMenuRequested.connect(self.onCustomContextMenuRequested)
+		self.customContextMenuRequested.connect(self._onCustomContextMenuRequested)
 		self.formatInfoContainer = None
 		self.optionsConfigKey = "RangeTree"
 		self.setMouseTracking(True)
 		self.setAcceptDrops(True)
-
-	formatInfoUpdated = pyqtSignal()
+		self.onlyPrintable = False
 
 	def wheelEvent(self, a0: QtGui.QWheelEvent) -> None:
 		super().wheelEvent(a0)
 		a0.accept()
 
-	def updateTree(self, fi_tree):
+	def updateTree(self, fi_trees):
 		self.clear()
-		if fi_tree is not None:
+		for fi_tree in fi_trees:
+			if fi_tree is None: continue
 			root = QTreeWidgetItem(self)
 			root.setExpanded(True)
 			if self.formatInfoContainer:
 				root.setText(0, self.formatInfoContainer.file_name)
-			fi_tree.addToTree(root)
+			fi_tree.addToTree(root, self.onlyPrintable)
 
-	def fiTreeItemActivated(self, item, column):
+	def _fiTreeItemActivated(self, item, column):
 		pass
 
 	def hilightFormatInfoTree(self, range):
@@ -100,7 +109,7 @@ class RangeTreeWidget(QTreeWidget):
 	#region Context Menu
 	################################################
 
-	def onCustomContextMenuRequested(self, point):
+	def _onCustomContextMenuRequested(self, point):
 		ctx = QMenu("Context menu", self)
 		item = self.itemAt(point)
 		if item != None:
@@ -110,7 +119,7 @@ class RangeTreeWidget(QTreeWidget):
 			if item.parent() != None:
 				parentSource = item.parent().data(0, Range.SourceDescRole)
 			if isinstance(source, FormatInfo):
-				if isinstance(source.fi, StructFI):
+				if isinstance(source.fi, (StructFI, UnionFI)):
 					ctx.addAction("Add field ...", lambda: self.addField(source, "StructField"))
 					ctx.addSeparator()
 				if isinstance(source.fi, VariantStructFI):
@@ -119,7 +128,7 @@ class RangeTreeWidget(QTreeWidget):
 				if isinstance(source.fi, SwitchFI):
 					ctx.addAction("Add case ...", lambda: self.addField(source, "SwitchItem"))
 					ctx.addSeparator()
-				if parentSource is not None and isinstance(parentSource, StructFI):
+				if parentSource is not None and isinstance(parentSource.fi, (StructFI, UnionFI, BitStructFI)):
 					ctx.addAction("Remove this field", lambda: self.removeField(parentSource, range.field_name))
 					ctx.addSeparator()
 				ctx.addAction("Edit ...", lambda: self.editField(source))
@@ -134,23 +143,25 @@ class RangeTreeWidget(QTreeWidget):
 		ctx.addAction("Load format info ...", self.fileOpenFormatInfo)
 		if self.formatInfoContainer and self.formatInfoContainer.file_name:
 			ctx.addAction("Save format info", lambda: self.saveFormatInfo(self.formatInfoContainer.file_name))
+		ctx.addAction(QAction(parent=ctx, text="Only printable", triggered=self._toggleOnlyPrintable, checkable=True, checked=self.onlyPrintable))
 		ctx.exec(self.mapToGlobal(point))
 
+	def _toggleOnlyPrintable(self):
+		self.onlyPrintable = not self.onlyPrintable
+
+
 	def addField(self, parent, typeName):
 		def ok(params):
 			parent.updateParams(children=parent.params['children']+[params])
-			self.formatInfoUpdated.emit()
-			self.saveFormatInfo(self.formatInfoContainer.file_name)
+			self._afterUpdate()
 
 		showTypeEditorDlg("format_info.tes", typeName, ok_callback=ok)
 
-
 	def removeField(self, parent, field_name):
 		ch = parent.params['children']
-		del ch[field_name]
+		del ch[next(i for i,(key,el) in enumerate(ch) if key == field_name)]
 		parent.updateParams(children=ch)
-		self.formatInfoUpdated.emit()
-
+		self._afterUpdate()
 
 	def editDisplayParams(self, parent):
 		params = showSettingsDlg([
@@ -163,8 +174,7 @@ class RangeTreeWidget(QTreeWidget):
 		if params.get("textcolor") == "": params["textcolor"] = None
 		if params.get("section") == "": params["section"] = None
 		parent.updateParams(**params)
-		self.formatInfoUpdated.emit()
-		self.saveFormatInfo(self.formatInfoContainer.file_name)
+		self._afterUpdate()
 
 	def editField(self, element: FormatInfo):
 		"""
@@ -176,27 +186,25 @@ class RangeTreeWidget(QTreeWidget):
 		#if ok:
 		def ok_callback(result):
 			element.from_text(result)
-			self.formatInfoUpdated.emit()
-			self.saveFormatInfo(self.formatInfoContainer.file_name)
+			self._afterUpdate()
 		showScintillaDialog(self, "Edit field", element.to_text(0, None), ok_callback=ok_callback)
 
-
 	def editField2(self, element: FormatInfo):
 		def ok_callback(params):
 			element.deserialize(params)
-			self.formatInfoUpdated.emit()
-			self.saveFormatInfo(self.formatInfoContainer.file_name)
+			self._afterUpdate()
 		showTreeEditorDlg("format_info.tes", "AnyFI", element.serialize(), ok_callback=ok_callback)
 
-
 	def repeatField(self, element: FormatInfo):
 		def ok_callback(params):
 			element.setContents(RepeatStructFI, params)
-			self.formatInfoUpdated.emit()
-			self.saveFormatInfo(self.formatInfoContainer.file_name)
+			self._afterUpdate()
 
 		showTypeEditorDlg("format_info.tes", "RepeatStructFI", { "children": element.serialize() }, ok_callback=ok_callback)
 
+	def _afterUpdate(self):
+		self.saveFormatInfo(self.formatInfoContainer.file_name)
+
 	#endregion
 
 	################################################
@@ -211,7 +219,6 @@ class RangeTreeWidget(QTreeWidget):
 		self.formatInfoContainer = InteractiveFormatInfoContainer(self, )
 		self.formatInfoContainer.load_from_string("DEFAULT struct(endianness="<") {}")
 		self.formatInfoContainer.file_name = fileName
-		self.formatInfoUpdated.emit()
 		self.saveFormatInfo(self.formatInfoContainer.file_name)
 
 	def fileOpenFormatInfo(self):
@@ -222,12 +229,11 @@ class RangeTreeWidget(QTreeWidget):
 
 	def loadFormatInfo(self, **loadSrc):
 		try:
-			self.formatInfoContainer = InteractiveFormatInfoContainer(self, **loadSrc)
+			self.formatInfoContainer = InteractiveFormatInfoContainer(**loadSrc)
 		except Exception as ex:
 			traceback.print_exc()
 			QMessageBox.warning(self, "Failed to parse format info description", str(ex))
 			return
-		self.formatInfoUpdated.emit()
 
 	def saveFormatInfo(self, fileName):
 		self.formatInfoContainer.write_file(fileName)
