@@ -14,7 +14,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
+import logging
 import uuid
 
 from PyQt5 import QtGui, QtCore, QtWidgets
@@ -22,14 +22,15 @@ from PyQt5.QtCore import (Qt, pyqtSignal, pyqtSlot, QEvent, QSize)
 from PyQt5.QtGui import QKeyEvent, QFont
 from PyQt5.QtWidgets import QWidget, QVBoxLayout, \
 	QFormLayout, QComboBox, QLineEdit, QCheckBox, QPushButton, QSizePolicy, QHBoxLayout, QLabel, \
-	QListWidget, QListWidgetItem, QFrame, QScrollArea, QDialog, QDoubleSpinBox, QSpinBox, QTableWidget, QMenu, \
+	QListWidget, QListWidgetItem, QFrame, QScrollArea, QDialog, QDoubleSpinBox, QSpinBox, QMenu, \
 	QInputDialog, QTreeWidget, QTreeWidgetItem, QMessageBox
 
 from pre_workbench.guihelper import makeDlgButtonBox
 from pre_workbench.structinfo.expr import Expression
 from pre_workbench.configs import respath
 from pre_workbench.structinfo import xdrm
-from pre_workbench.genericwidgets import MdiFile, showWidgetDlg
+from pre_workbench.controls.genericwidgets import showWidgetDlg
+from pre_workbench.windows.mdifile import MdiFile
 from pre_workbench.typeregistry import WindowTypes, DataWidgetTypes
 
 FILE_MAGIC = b"\xde\xca\xf9\x30"
@@ -277,8 +278,10 @@ class StructTypeEditor(StructuredTypeEditor):
 			self.layout().addRow(labelWidget, child)
 			if ("uiShowIf" in field): self.conditionals.append((Expression(expr_str=field['uiShowIf']), labelWidget, child));
 			# TODO fix autoincrement fields
-			#if ("uiFlags" in field and (field['uiFlags'] & Field_UiFlags_autoIncrement) > 0 and parent.children):
-			#	child.set(parent.children.reduce(function(p,c) { return Math.max(p,c.getFieldValue(field.name))}, 0) + 1);
+			if ("uiFlags" in field and (field['uiFlags'] & Field_UiFlags_autoIncrement) > 0 and hasattr(self.parent(), "get")):
+				siblings = self.parent().get()
+				if siblings is not None and type(siblings) == list and len(siblings) > 0:
+					child.set(max(sib.get(field['name'], 0) for sib in siblings) + 1)
 			if ("uiFlags" in field and (field['uiFlags'] & Field_UiFlags_advanced) > 0):
 				self.advancedElements.append((labelWidget, child))
 
@@ -421,7 +424,6 @@ class ListTypeEditor(StructuredTypeEditor):
 			self.add(item)
 
 
-
 class ListTypeEditorItem(QWidget):
 	def __init__(self, body):
 		super().__init__()
@@ -443,42 +445,6 @@ class ListTypeEditorItem(QWidget):
 		self.deleteLater()
 
 
-class TypeEditorFileWindow(QScrollArea, MdiFile):
-	def __init__(self, **params):
-		super().__init__()
-		self.params = params
-		self.initUI()
-		self.initMdiFile(params.get("fileName"), params.get("isUntitled", False), type(self).patterns, "untitled%d" + type(self).fileExts[0])
-	def sizeHint(self):
-		return QSize(600,400)
-	def initUI(self):
-		self.setStyleSheet("StructuredTypeEditor { border: 1px solid #bbb }")
-		self.metaSchema = TypeEditorSchema(open(type(self).schema,'rb').read())
-		#self.editor = self.metaSchema.generateTypeEditorByName(self, type(self).typeName)
-		self.editor = JsonView(schema=self.metaSchema, rootTypeDefinition=[Type_Named,type(self).typeName])
-		self.setWidget(self.editor)
-		self.setWidgetResizable(True)
-	def loadFile(self, fileName):
-		self.editor.deserialize(open(fileName,'rb').read())
-		self.setCurrentFile(fileName)
-		self.editor.updated.connect(self.documentWasModified)
-	def saveFile(self, fileName):
-		with open(fileName, "wb") as f:
-			f.write(self.editor.serialize())
-		self.setCurrentFile(fileName)
-		return True
-
-
-
-@WindowTypes.register(fileExts=['.tes'], schema=respath('meta_schema.tes'), typeName='Interface', description='Type Editor Schema', patterns='Type Editor Schema (*.pfi)')
-class TypeEditorSchemaFileWindow(TypeEditorFileWindow):
-	pass
-
-@WindowTypes.register(fileExts=['.pfi'], schema=respath('format_info.tes'), typeName='FormatInfoFile', description='Protocol Format Info Specification')
-class ProtocolFormatInfoFileWindow(TypeEditorFileWindow):
-	pass
-
-
 def resolveSchema(schemaFile):
 	if isinstance(schemaFile, TypeEditorSchema): return schemaFile
 	if isinstance(schemaFile, str): return TypeEditorSchema(open(respath(schemaFile),'rb').read())
@@ -500,7 +466,6 @@ def showTypeEditorDlg(schemaFile, typeName, values=None, title="Editor", ok_call
 	return editor.get()
 
 
-
 def showTreeEditorDlg(schemaFile, typeName, values=None, title="Editor", ok_callback=None):
 	dlg = QDialog()
 	dlg.setWindowTitle(title)
@@ -596,7 +561,9 @@ class JsonView(QTreeWidget):
 
 			if typ is list:
 				ctx.addSeparator()
-				ctx.addAction("Add item ...", lambda: self.addField(item, "AnyFI"))
+				childTypeDef = item.data(1, QtCore.Qt.UserRole + 2)
+				if childTypeDef:
+					ctx.addAction("Add item ...", lambda: self.addField(item, None, childTypeDef))
 				ctx.addAction("Clear list", lambda: self.clearList(item)).setEnabled(False)
 			if parentTyp is dict:
 				ctx.addSeparator()
@@ -632,7 +599,7 @@ class JsonView(QTreeWidget):
 
 	def editField(self, item):
 		typeDef = item.data(1, QtCore.Qt.UserRole + 1)
-		print("typeDef",typeDef)
+		logging.debug("editField: typeDef %r",typeDef)
 		data = self.tree_fetch(item)
 		if typeDef is not None:
 			showTypeEditorDlg(self.schema, typeDef, data, "Edit item", ok_callback=lambda res: self.tree_set_row(item, res, typeDef))
@@ -646,11 +613,22 @@ class JsonView(QTreeWidget):
 			res, ok = QInputDialog.getDouble(self, "Edit", "Edit float value", value=data)
 			if ok: self.tree_set_row(item, res, typeDef)
 		elif isinstance(data, (bytes, bytearray)):
-			from pre_workbench.hexview import showHexView2Dialog
+			from pre_workbench.controls.hexview import showHexView2Dialog
 			showHexView2Dialog(self, "Edit bytes value", data, lambda newVal: self.tree_set_row(item, newVal, typeDef))
 		else:
 			QMessageBox.information(self, "Note", "Editor for data type "+str(type(data))+" not implemented")
 
+	def addField(self, parentItem, name, typeDef):
+		parentTypeDef = parentItem.data(1, QtCore.Qt.UserRole + 1)
+		parentData = self.tree_fetch(parentItem)
+		def ok(res):
+			if name is None:
+				parentData.append(res)
+			else:
+				parentData[name] = res
+			self.tree_set_row(parentItem, parentData, parentTypeDef)
+		showTypeEditorDlg(self.schema, typeDef, None, "Add field", ok_callback=ok)
+
 	def removeField(self, item):
 		item.parent().removeChild(item)
 
@@ -692,7 +670,7 @@ class JsonView(QTreeWidget):
 		return me
 
 	def tree_set_row(self, me, val, origtypeDef):
-		print(origtypeDef)
+		logging.debug("tree_set_row: origTypeDef = %r", origtypeDef)
 		me.setData(1, QtCore.Qt.UserRole, type(val))
 		me.setData(1, QtCore.Qt.UserRole + 1, origtypeDef)
 		if self.schema is not None and origtypeDef is not None:
@@ -712,13 +690,13 @@ class JsonView(QTreeWidget):
 		for i in reversed(range(me.childCount())):
 			me.removeChild(me.child(i))
 
-		print("typeDef:",typeDef)
+		logging.debug("tree_set_row: typeDef = %r",typeDef)
 		if isinstance(val, dict):
 			if typeDef is None:
 				childTypeDefs = {}
 			else:
 				childTypeDefs = {field['name'] : field['type'] for field in typeDef[1]['fields']}
-			print("defs",childTypeDefs)
+			logging.debug("tree_set_row: defs = %r",childTypeDefs)
 			for key, cc in val.items():
 				self.tree_add_row(key, cc, me, childTypeDefs.pop(key, None))
 			me.setData(1, QtCore.Qt.UserRole + 2, childTypeDefs)
@@ -732,9 +710,11 @@ class JsonView(QTreeWidget):
 			choiceItem = getChoiceSubtypeById(typeDef, val)
 			if choiceItem is not None:
 				me.setData(1, QtCore.Qt.UserRole + 2, val[0])
-				print("choiceItem",choiceItem)
+				logging.debug("tree_set_row: choiceItem = %r",choiceItem)
 				self.tree_add_row(choiceItem['name'], val[1], me, choiceItem['type'])
 			else:
+				if typeKind == Type_List:
+					me.setData(1, QtCore.Qt.UserRole + 2, typeDef[1]['itemType'])
 				for i, cc in enumerate(val):
 					key = str(i)
 					self.tree_add_row(key, cc, me, typeDef[1]['itemType'] if typeKind == Type_List else None)
